﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TrabalhoLib.Classes.Relatorios;
using TrabalhoLib.Excecoes;

namespace TrabalhoLib.Classes
{
    public partial class EleicaoMain
    {

        public EleicaoMain()
        {
            this.candidatos = new List<Candidato>();
            this.partidos = new List<Partido>();
        }

        public List<dynamic> GetRelatorio()
        {

            int qtdVagas = 0;

            try
            {

                string line;
                string[] lineSplit = new string[] { };

                string path = @"C:\Users\elson\Documents\Elson Jr\LP\drive-download-20181001T212928Z-001\testes\vitoria\in\divulga.csv";

                using (StreamReader rd = new StreamReader(path, Encoding.UTF7))
                {
                    while ((line = rd.ReadLine()) != null)
                    {
                        lineSplit = line.Split(';');

                        if (lineSplit.FirstElement().StartsWith("*"))
                        {
                            int indexToRemove = lineSplit[(int)LinhasCSV.SEQUENCIA].IndexOf('*');
                            lineSplit[(int)LinhasCSV.SEQUENCIA] = lineSplit[(int)LinhasCSV.SEQUENCIA].Remove(indexToRemove);

                            qtdVagas++; // Acrescenta o numero de vagas preenchidas

                            this.ManagmentInfo(lineSplit, StatusCandidato.ELEITO);

                        }

                        else if (lineSplit.FirstElement().StartsWith("#"))
                        {
                            lineSplit[0] = lineSplit[0].Remove(lineSplit[0].IndexOf('#'));
                            Console.WriteLine("Tem cerquilha");
                        }

                        // O out do TryParse representa que a variavel foi passada por referência
                        else if (int.TryParse(lineSplit.FirstElement(), out int numero))
                        {

                            this.ManagmentInfo(lineSplit, StatusCandidato.NAO_ELEITO);

                        }
                    }
                }

            }
            catch (FileNotFoundException ex)
            {
                Console.WriteLine("Não foi possível encontrar o arquivo!");
            }
            catch (DirectoryNotFoundException ex)
            {
                Console.WriteLine("Diretório especificado não encotrado!");
            }
            catch (IOException ex)
            {
                Console.WriteLine("É possível que o arquivo esteja aberto. Por favor, feche-o!");
            }
            catch (EmptyArrayException ex)
            {
                Console.WriteLine(ex.Message);
            }
            finally
            {

                //Console.WriteLine(new Relatorio().GetCandidatosEleitos(candidatos, "Candidatos eleitos"));

                //Console.WriteLine(Environment.NewLine + Environment.NewLine);

                //Console.WriteLine(new Relatorio().GetCandidatosMaisVotados(candidatos, 4));

                //Console.WriteLine(new Relatorio(qtdVagas, candidatos).GetCandidatosBeneficiados());

                //Print(partidos);
            }

            List<dynamic> v = new List<dynamic>();

            Relatorio r = new Relatorio(qtdVagas, candidatos);

            v.Add(r.RelatorioCandidatosEleitos);
            v.Add(r.CandidatosMaisVotados);

            return v;
        }

        private void ManagmentInfo(string[] lineSplit, StatusCandidato status)
        {
            InfoCSV info = new InfoCSV(lineSplit);

            // Verifica se o partido ja foi criado
            if (this.VerifyPartido(partidos, info.SiglaPartido))
            {
                Partido p = this.partidos.Find(element => element.Sigla.Equals(info.SiglaPartido));

                p.UpdateQtdVotos(info.Votacao);

                Candidato c = new Candidato(info.Candidato, info.Numero, info.Votacao, info.Validos, p, status);

                this.candidatos.Add(c);

                p.AddCandidato(c);

                int index = this.partidos.FindIndex(element => element.Sigla.Equals(info.SiglaPartido));

                this.partidos[index] = p;

            }

            else
            {
                List<Partido> coligacao = new List<Partido>();

                this.FillColigacao(coligacao, info.Coligacao);

                Partido partido = new Partido(info.SiglaPartido, info.Votacao, coligacao);

                Candidato candidato = new Candidato(info.Candidato, info.Numero, info.Votacao, info.Validos, partido, status);

                this.candidatos.Add(candidato);

                partido.AddCandidato(candidato);

                this.partidos.Add(partido);
            }
        }

        private void FillColigacao(List<Partido> coligacao, string[] partidos)
        {
            if (partidos != null)
            {
                foreach (string p in partidos)
                {
                    coligacao.Add(new Partido(p));
                }
            }
        }

        private void Print(List<Partido> partidos)
        {
            foreach (Partido p in partidos)
            {
                Console.WriteLine("Partidos");
                Console.WriteLine("**** SIGLA: " + p.Sigla);
                Console.WriteLine("**** VOTOS: " + p.QtdVotos);
                foreach (Candidato c in p.Candidatos)
                {
                    Console.WriteLine("**** CANDIDATO: " + c.Nome);
                }
                foreach (Partido c in p.Coligacao)
                {
                    Console.WriteLine("**** COLIGACAO: " + c.Sigla);
                }
            }
        }

        private bool VerifyPartido(List<Partido> partidos, string p)
        {
            return partidos.Exists(partido => partido.Sigla.Equals(p));
        }

    }
}
