﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TrabalhoLib.Classes
{
    public partial class InfoCSV
    {

        public InfoCSV(string[] linha)
        {
            int.TryParse(linha[(int)LinhasCSV.NUMERO], out int n);
            this.Numero = n;

            this.Candidato = linha[(int)LinhasCSV.CANDIDATO];

            this.SiglaPartido = GetSigla(linha[(int)LinhasCSV.PARTIDO_COLIGACAO]);

            this.Coligacao = GetColigacao(linha[(int)LinhasCSV.PARTIDO_COLIGACAO]);

            this.Votacao = GetVotacao(linha[(int)LinhasCSV.VOTACAO]);

            this.Validos = GetValidos(linha[(int)LinhasCSV.VALIDOS]);
        }

        private double GetValidos(string linhaValidos)
        {
            return linhaValidos.ParsePercentageStringToDouble();
        }

        private int GetVotacao(string linhaVotacao)
        {
            int.TryParse(linhaVotacao, NumberStyles.Number, CultureInfo.CurrentCulture, out int votos);

            return votos;
        }

        private string[] GetColigacao(string linhaPartido)
        {
            linhaPartido = linhaPartido.Replace(" ", "");

            if (ExisteColigacao(linhaPartido))
            {
                // Pega todos os partidos da coligação
                linhaPartido = linhaPartido.Substring(linhaPartido.IndexOf('-') + 1);

                string[] coligacao = new string[0];

                foreach (string sigla in linhaPartido.Split('/'))
                {
                    if (!this.siglaPartido.Equals(sigla))
                    {
                        // Array em C# é imutavel (arrays como string[], int[], double[], etc
                        // Ou seja, não podem mudar de tamanho
                        // Assim, usou-se o método Array.Resize para aumentar o tamanho em 1
                        // E então acrescentou um elemento na ultima posição
                        Array.Resize(ref coligacao, coligacao.Length + 1);

                        // Ex.:
                        // Tamanho 1, insere na posição 0
                        // Tamanho 2, insere na posição 1
                        coligacao[coligacao.Length - 1] = sigla;
                    }
                }

                return coligacao;
            }

            return null;

        }

        private string GetSigla(string linhaPartido)
        {

            linhaPartido = linhaPartido.Replace(" ", "");

            if (ExisteColigacao(linhaPartido))
            {
                return linhaPartido.Substring(0, linhaPartido.IndexOf('-'));
            }

            return linhaPartido;
        }

        private bool ExisteColigacao(string textoPartido)
        {
            if (textoPartido.Contains("-"))
            {
                return true;
            }

            return false;
        }

    }
}
