﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TrabalhoLib.Classes.Relatorios
{
    public partial class Relatorio
    {

        public Relatorio(int qtdVagas, List<Candidato> candidatos)
        {

            this.qtdVagas = qtdVagas;

            this.candidatosEleitos = candidatos
                .FindAll(candidato => candidato.Situacao == StatusCandidato.ELEITO);

            this.candidatosMaisVotados = candidatos
                .OrderByDescending(candidato => candidato.QtdVotos)
                .ToList();//.GetRange(0, this.qtdVagas);

            this.relatorioCandidatosEleitos = new RelatorioCandidatosEleitos().GetCandidatosEleitos(this.candidatosEleitos);

        }

        public string GetCandidatosMaisVotados(List<Candidato> candidatos, int qtdVagas)
        {
            const string titulo = "Candidatos mais votados (em ordem decrescente de votação e respeitando número de vagas):";

            int posicao = 1;
            string result = titulo + Environment.NewLine;

            List<Candidato> candidatosMaisVotados = candidatos
                .OrderByDescending(candidato => candidato.QtdVotos)
                .ToList()
                .GetRange(0, qtdVagas);

            foreach (Candidato candidato in candidatosMaisVotados)
            {

                string nome = candidato.Nome;
                string partido = candidato.Partido.Sigla;
                string votos = candidato.QtdVotos.ToString();

                result += (posicao++) + " - " + nome + " (" + partido + ", " + votos + " votos)";

                if (candidato.Partido.Coligacao.Count != 0)
                {

                    string coligacao = partido;

                    foreach (Partido c in candidato.Partido.Coligacao)
                    {
                        coligacao += " / " + c.Sigla;
                    }

                    result += " - Coligação: " + coligacao;

                }

                result += Environment.NewLine;

            }

            return result;

        }

        public void GetCandidatosNaoBeneficiados()
        {

            List<Candidato> c = CandidatosMaisVotados
                .GetRange(0, 15)
                .Except(candidatosEleitos)
                .ToList();

            foreach (Candidato ca in c)
            {
                Console.WriteLine( (candidatosMaisVotados.IndexOf(ca) + 1) + " - " + ca.Nome);
            }

        }

        public string GetCandidatosBeneficiados()
        {
            List<Candidato> candidatos = candidatosEleitos
                .Except(candidatosMaisVotados.GetRange(0, 15))
                .ToList();

            string result = "";

            foreach (Candidato c in candidatos)
            {
                result += (candidatosMaisVotados.IndexOf(c) + 1) + " - " + c.Nome + " (" + c.Partido.Sigla + ", " + c.QtdVotos + " votos)";


                if (c.Partido.Coligacao.Count != 0)
                {
                    result += " - Coligação: " + c.Partido.Sigla;

                    foreach (Partido partido in c.Partido.Coligacao)
                    {
                        result += " / " + partido.Sigla;
                    }

                }

                result += Environment.NewLine;

            }

            return result;

        }

    }
}
